@@ -399,7 +399,7 @@ class BrowserController {
   int ping_timer_watch_;
   bool ping_flag_;
   std::string up_buffer_;
-  typedef std::map<size_t, BrowserElementImpl *> BrowserElements;
+  typedef LightMap<size_t, BrowserElementImpl *> BrowserElements;
   BrowserElements browser_elements_;
   size_t browser_seq_;
   bool removing_watch_;
@@ -564,16 +564,54 @@ class BrowserElementImpl {
     Slot *to_string_;
   };
 
+  // Wraps a host method into a ScriptableInterface object, while doesn't
+  // take the ownership of the method slot.
+  class HostSlotWrapper : public ScriptableHelperDefault {
+   public:
+    DEFINE_CLASS_ID(0xc12afe14b57d4e0b, ScriptableInterface);
+    HostSlotWrapper(ScriptableInterface *parent, const std::string &name)
+        : parent_(parent), name_(name) {
+    }
+
+    virtual PropertyType GetPropertyInfo(const char *name,
+                                         Variant *prototype) {
+      if (!*name) {
+        if (prototype)
+          *prototype = Variant(GetSlot());
+        return PROPERTY_METHOD;
+      }
+      return PROPERTY_NOT_EXIST;
+    }
+
+    virtual ResultVariant GetProperty(const char *name) {
+      return ResultVariant(!*name ? Variant(GetSlot()) : Variant());
+    }
+
+    Slot *GetSlot() {
+      ScriptableInterface *parent_obj = parent_.Get();
+      if (parent_obj) {
+        ResultVariant method = parent_obj->GetProperty(name_.c_str());
+        if (method.v().type() == Variant::TYPE_SLOT)
+          return VariantValue<Slot *>()(method.v());
+      }
+      return NULL;
+    }
+
+    ScriptableHolder<ScriptableInterface> parent_;
+    std::string name_;
+  };
+
   BrowserElementImpl(BrowserElement *owner)
       : owner_(owner),
         object_seq_(0),
         controller_(BrowserController::get()),
         browser_id_(0),
         content_type_("text/html"),
-        content_updated_(false),
         socket_(NULL),
         x_(0), y_(0), width_(0), height_(0),
+        content_updated_(false),
         minimized_(false), popped_out_(false),
+        always_open_new_window_(true),
         minimized_connection_(owner->GetView()->ConnectOnMinimizeEvent(
             NewSlot(this, &BrowserElementImpl::OnViewMinimized))),
         restored_connection_(owner->GetView()->ConnectOnRestoreEvent(
@@ -586,6 +624,10 @@ class BrowserElementImpl {
             NewSlot(this, &BrowserElementImpl::OnViewDockUndock))),
         undock_connection_(owner->GetView()->ConnectOnUndockEvent(
             NewSlot(this, &BrowserElementImpl::OnViewDockUndock))) {
+  }
+
+  ~BrowserElementImpl() {
+    Deactivate();
     minimized_connection_->Disconnect();
     restored_connection_->Disconnect();
     popout_connection_->Disconnect();
@@ -594,10 +636,6 @@ class BrowserElementImpl {
     undock_connection_->Disconnect();
   }
 
-  ~BrowserElementImpl() {
-    Deactivate();
-  }
-
   void Deactivate() {
     if (browser_id_) {
       // If socket_ is not a valid socket, the child may have closed the
@@ -674,6 +712,10 @@ class BrowserElementImpl {
       impl->controller_->SendCommand(kNewBrowserCommand, impl->browser_id_,
                                      socket_id_str.c_str(), NULL);
       impl->UpdateChildContent();
+      impl->controller_->SendCommand(kSetAlwaysOpenNewWindowCommand,
+                                     impl->browser_id_,
+                                     impl->always_open_new_window_ ? "1" : "0",
+                                     NULL);
     }
   }
 
@@ -783,6 +825,8 @@ class BrowserElementImpl {
         return StringPrintf("hobj %zu", AddHostObject(obj));
       }
       case Variant::TYPE_SLOT: {
+        // Note: this case is not for the result of GetProperty() of a host
+        // object which is specially handled in GetHostObjectProperty().
         Slot *slot = VariantValue<Slot *>()(value);
         if (!slot) return kNullStr;
         ScriptableFunction *obj = new ScriptableFunction(slot);
@@ -920,9 +964,19 @@ class BrowserElementImpl {
     if (!object)
       return StringPrintf("exception: host object %s not found", object_id_str);
     std::string property_name;
+    ResultVariant result;
     if (DecodeJavaScriptString(property, &property_name))
-      return EncodeValue(object->GetProperty(property_name.c_str()).v());
-    return EncodeValue(object->GetPropertyByIndex(atoi(property)).v());
+      result = object->GetProperty(property_name.c_str());
+    else
+      result = object->GetPropertyByIndex(atoi(property));
+
+    if (result.v().type() == Variant::TYPE_SLOT) {
+      // Specially handle Slot property, don't wrap it into a
+      // ScriptableFunction because it is owned by the object.
+      return EncodeValue(Variant(new HostSlotWrapper(object, property_name)));
+    } else {
+      return EncodeValue(result.v());
+    }
   }
 
   std::string SetHostObjectProperty(const char *object_id_str,
@@ -1033,7 +1087,22 @@ class BrowserElementImpl {
         LOG("%s feedback needs 3 parameters, but only %zu is given",
             kOpenURLFeedback, param_count);
       } else {
-        result = OpenURL(params[2]) ? '1' : '0';
+        result = ongotourl_signal_(params[2], true) || OpenURL(params[2]) ?
+            '1' : '0';
+      }
+    } else if (strcmp(type, kGoToURLFeedback) == 0) {
+      if (param_count != 3) {
+        LOG("%s feedback needs 3 parameters, but only %zu is given",
+            kGoToURLFeedback, param_count);
+      } else {
+        result = ongotourl_signal_(params[2], false) ? '1' : '0';
+      }
+    } else if (strcmp(type, kNetErrorFeedback) == 0) {
+      if (param_count != 3) {
+        LOG("%s feedback needs 3 parameters, but only %zu is given",
+            kNetErrorFeedback, param_count);
+      } else {
+        result = onerror_signal_(params[2]) ? '1' : '0';
       }
     } else {
       LOG("Unknown feedback: %s", type);
@@ -1041,9 +1110,19 @@ class BrowserElementImpl {
     return result;
   }
 
-  typedef std::map<size_t, ScriptableHolder<ScriptableInterface> >
+  void SetAlwaysOpenNewWindow(bool always_open_new_window) {
+    if (always_open_new_window_ != always_open_new_window) {
+      always_open_new_window_ = always_open_new_window;
+      if (browser_id_) {
+        controller_->SendCommand(kSetAlwaysOpenNewWindowCommand, browser_id_,
+                                 always_open_new_window ? "1" : "0", NULL);
+      }
+    }
+  }
+
+  typedef LightMap<size_t, ScriptableHolder<ScriptableInterface> >
       HostObjectMap;
-  typedef std::map<size_t, BrowserObjectWrapper *> BrowserObjectMap;
+  typedef LightMap<size_t, BrowserObjectWrapper *> BrowserObjectMap;
   HostObjectMap host_objects_;
   BrowserObjectMap browser_objects_;
 
@@ -1054,15 +1133,18 @@ class BrowserElementImpl {
 
   std::string content_type_;
   std::string content_;
-  bool content_updated_;
   GtkWidget *socket_;
   gint x_, y_, width_, height_;
-  bool minimized_;
-  bool popped_out_;
+  bool content_updated_ : 1;
+  bool minimized_ : 1;
+  bool popped_out_ : 1;
+  bool always_open_new_window_ : 1;
   ScriptableHolder<ScriptableInterface> external_object_;
   Connection *minimized_connection_, *restored_connection_,
              *popout_connection_, *popin_connection_,
              *dock_connection_, *undock_connection_;
+  Signal2<bool, const char *, bool> ongotourl_signal_;
+  Signal1<bool, const char *> onerror_signal_;
 };
 
 BrowserController *BrowserController::instance_ = NULL;
@@ -1099,14 +1181,10 @@ void BrowserController::ProcessFeedback(size_t param_count,
   }
 }
 
-class BrowserElement::Impl : public BrowserElementImpl {
- public:
-  Impl(BrowserElement *owner) : BrowserElementImpl(owner) { }
-};
-
 BrowserElement::BrowserElement(View *view, const char *name)
     : BasicElement(view, "browser", name, true),
-      impl_(new Impl(this)) {
+      impl_(new BrowserElementImpl(this)) {
+  SetEnabled(true);
 }
 
 void BrowserElement::DoClassRegister() {
@@ -1118,6 +1196,13 @@ void BrowserElement::DoClassRegister() {
                    NewSlot(&BrowserElement::SetContent));
   RegisterProperty("external", NULL,
                    NewSlot(&BrowserElement::SetExternalObject));
+  RegisterProperty("alwaysOpenNewWindow",
+                   NewSlot(&BrowserElement::IsAlwaysOpenNewWindow),
+                   NewSlot(&BrowserElement::SetAlwaysOpenNewWindow));
+  RegisterClassSignal("onerror", &BrowserElementImpl::onerror_signal_,
+                      &BrowserElement::impl_);
+  RegisterClassSignal("ongotourl", &BrowserElementImpl::ongotourl_signal_,
+                      &BrowserElement::impl_);
 }
 
 BrowserElement::~BrowserElement() {
@@ -1154,5 +1239,13 @@ BasicElement *BrowserElement::CreateInstance(View *view, const char *name) {
   return new BrowserElement(view, name);
 }
 
+bool BrowserElement::IsAlwaysOpenNewWindow() const {
+  return impl_->always_open_new_window_;
+}
+
+void BrowserElement::SetAlwaysOpenNewWindow(bool always_open_new_window) {
+  impl_->SetAlwaysOpenNewWindow(always_open_new_window);
+}
+
 } // namespace gtkmoz
 } // namespace ggadget
