@@ -356,7 +356,16 @@ CanvasInterface::Alignment QtEditElement::GetAlign() const {
   return CanvasInterface::ALIGN_LEFT;
 }
 
-void QtEditElement::SetAlign(CanvasInterface::Alignment align) {
+void QtEditElement::SetAlign(CanvasInterface::Alignment) {
+  // TODO
+}
+
+CanvasInterface::VAlignment QtEditElement::GetVAlign() const {
+  // TODO
+  return CanvasInterface::VALIGN_TOP;
+}
+
+void QtEditElement::SetVAlign(CanvasInterface::VAlignment) {
   // TODO
 }
 
