@@ -16,11 +16,11 @@
 
 #include "converter.h"
 
-#include <cmath>
 #include <jsobj.h>
 #include <jscntxt.h>
 #include <jsfun.h>
 #include <jsnum.h>
+#include <cmath>
 #include <ggadget/scriptable_array.h>
 #include <ggadget/scriptable_binary_data.h>
 #include <ggadget/scriptable_holder.h>
@@ -41,6 +41,8 @@ namespace smjs {
 
 static JSBool ConvertJSToNativeVoid(JSContext *cx, jsval js_val,
                                     Variant *native_val) {
+  GGL_UNUSED(cx);
+  GGL_UNUSED(js_val);
   *native_val = Variant();
   return JS_TRUE;
 }
@@ -137,6 +139,12 @@ static JSBool ConvertJSToNativeString(JSContext *cx, jsval js_val,
       return JS_TRUE;
     }
   }
+
+  // Protects the result from JS_ValueToString from GC.
+  AutoLocalRootScope local_root_scope(cx);
+  if (!local_root_scope.good())
+    return JS_FALSE;
+
   JSString *js_string = JS_ValueToString(cx, js_val);
   if (js_string) {
     jschar *chars = JS_GetStringChars(js_string);
@@ -164,6 +172,12 @@ static JSBool ConvertJSToNativeUTF16String(JSContext *cx, jsval js_val,
     *native_val = Variant(kEmptyUTF16String);
     return JS_TRUE;
   }
+
+  // Protects the result from JS_ValueToString from GC.
+  AutoLocalRootScope local_root_scope(cx);
+  if (!local_root_scope.good())
+    return JS_FALSE;
+
   JSString *js_string = JS_ValueToString(cx, js_val);
   if (js_string) {
     jschar *chars = JS_GetStringChars(js_string);
@@ -210,20 +224,24 @@ static JSBool ConvertJSToSlot(JSContext *cx, NativeJSWrapper *owner,
       (JSVAL_IS_INT(js_val) && JSVAL_TO_INT(js_val) == 0)) {
     function_object = NULL;
   } else if (JSVAL_IS_STRING(js_val)) {
+    // Protects the result of CompileFunction from GC.
+    AutoLocalRootScope local_root_scope(cx);
+    if (!local_root_scope.good())
+      return JS_FALSE;
+
     JSString *script_source = JSVAL_TO_STRING(js_val);
     jschar *script_chars = JS_GetStringChars(script_source);
     if (!script_chars)
       return JS_FALSE;
 
-    std::string utf8_script;
-    ConvertStringUTF16ToUTF8(script_chars, JS_GetStringLength(script_source),
-                             &utf8_script);
-
     std::string filename;
     int lineno;
     JSScriptContext::GetCurrentFileAndLine(cx, &filename, &lineno);
-    JSFunction *function = CompileFunction(cx, utf8_script.c_str(),
-                                           filename.c_str(), lineno);
+    JSFunction *function = CompileFunction(cx,
+        UTF16ToUTF8Converter(script_chars,
+                             JS_GetStringLength(script_source)).get(),
+        filename.c_str(), lineno);
+
     if (!function)
       result = JS_FALSE;
     function_object = JS_GetFunctionObject(function);
@@ -475,6 +493,8 @@ JSBool ConvertJSArgsToNative(JSContext *cx, NativeJSWrapper *owner,
 static JSBool ConvertNativeToJSVoid(JSContext *cx,
                                     const Variant &native_val,
                                     jsval *js_val) {
+  GGL_UNUSED(cx);
+  GGL_UNUSED(native_val);
   *js_val = JSVAL_VOID;
   return JS_TRUE;
 }
@@ -482,6 +502,7 @@ static JSBool ConvertNativeToJSVoid(JSContext *cx,
 static JSBool ConvertNativeToJSBool(JSContext *cx,
                                     const Variant &native_val,
                                     jsval *js_val) {
+  GGL_UNUSED(cx);
   *js_val = BOOLEAN_TO_JSVAL(VariantValue<bool>()(native_val));
   return JS_TRUE;
 }
@@ -592,6 +613,9 @@ static JSBool ConvertNativeUTF16ToJSString(JSContext *cx,
 
 static JSBool ReturnSelf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                          jsval *rval) {
+  GGL_UNUSED(cx);
+  GGL_UNUSED(argc);
+  GGL_UNUSED(argv);
   *rval = OBJECT_TO_JSVAL(obj);
   return JS_TRUE;
 }
@@ -681,6 +705,9 @@ static JSBool ConvertNativeToJSDate(JSContext *cx,
 static JSBool ConvertNativeToJSFunction(JSContext *cx,
                                         const Variant &native_val,
                                         jsval *js_val) {
+  GGL_UNUSED(cx);
+  GGL_UNUSED(native_val);
+  GGL_UNUSED(js_val);
   DLOG("Reading native function in JavaScript");
   // Just leave the value that SpiderMonkey recorded in SetProperty.
   return JS_TRUE;
