@@ -171,6 +171,7 @@ JSBool NativeJSWrapper::CheckNotDeleted() {
 JSBool NativeJSWrapper::CallWrapperSelf(JSContext *cx, JSObject *obj,
                                         uintN argc, jsval *argv,
                                         jsval *rval) {
+  GGL_UNUSED(obj);
   if (JS_IsExceptionPending(cx))
     return JS_FALSE;
   // In this case, the real self object being called is at argv[-2].
@@ -197,6 +198,8 @@ JSBool NativeJSWrapper::CallWrapperMethod(JSContext *cx, JSObject *obj,
 JSBool NativeJSWrapper::WrapperDefaultToString(JSContext *cx, JSObject *obj,
                                                uintN argc, jsval *argv,
                                                jsval *rval) {
+  GGL_UNUSED(argc);
+  GGL_UNUSED(argv);
   if (JS_IsExceptionPending(cx))
     return JS_FALSE;
   NativeJSWrapper *wrapper = GetWrapperFromJS(cx, obj);
@@ -293,6 +296,7 @@ void NativeJSWrapper::FinalizeWrapper(JSContext *cx, JSObject *obj) {
 }
 
 uint32 NativeJSWrapper::MarkWrapper(JSContext *cx, JSObject *obj, void *arg) {
+  GGL_UNUSED(arg);
   // The current context may be different from wrapper's context during
   // GC marking.
   NativeJSWrapper *wrapper = GetWrapperFromJS(cx, obj);
@@ -412,9 +416,6 @@ JSBool NativeJSWrapper::CallMethod(uintN argc, jsval *argv, jsval *rval) {
 JSBool NativeJSWrapper::CallNativeSlot(const char *name, Slot *slot,
                                        uintN argc, jsval *argv, jsval *rval) {
   ASSERT(scriptable_);
-  AutoLocalRootScope local_root_scope(js_context_);
-  if (!local_root_scope.good())
-    return JS_FALSE;
 
   Variant *params = NULL;
   uintN expected_argc = argc;
@@ -474,10 +475,6 @@ JSBool NativeJSWrapper::GetPropertyByIndex(jsval id, jsval *vp) {
     // Should not occur.
     return JS_FALSE;
 
-  AutoLocalRootScope local_root_scope(js_context_);
-  if (!local_root_scope.good())
-    return JS_FALSE;
-
   int int_id = JSVAL_TO_INT(id);
   ResultVariant return_value = scriptable_->GetPropertyByIndex(int_id);
   if (!ConvertNativeToJS(js_context_, return_value.v(), vp)) {
@@ -496,10 +493,6 @@ JSBool NativeJSWrapper::SetPropertyByIndex(jsval id, jsval js_val) {
     // Should not occur.
     return JS_FALSE;
 
-  AutoLocalRootScope local_root_scope(js_context_);
-  if (!local_root_scope.good())
-    return JS_FALSE;
-
   int int_id = JSVAL_TO_INT(id);
   // Use the original value as the prototype, so that we can convert the
   // JS value to proper native type.
@@ -550,15 +543,10 @@ JSBool NativeJSWrapper::GetPropertyByName(jsval id, jsval *vp) {
   if (!idstr)
     return JS_FALSE;
 
-  AutoLocalRootScope local_root_scope(js_context_);
-  if (!local_root_scope.good())
-    return JS_FALSE;
-
   const jschar *utf16_name = JS_GetStringChars(idstr);
   size_t name_length = JS_GetStringLength(idstr);
-  std::string name;
-  ConvertStringUTF16ToUTF8(utf16_name, name_length, &name);
-  ResultVariant return_value = scriptable_->GetProperty(name.c_str());
+  UTF16ToUTF8Converter utf8_name(utf16_name, name_length);
+  ResultVariant return_value = scriptable_->GetProperty(utf8_name.get());
   if (!CheckException(js_context_, scriptable_))
     return JS_FALSE;
 
@@ -573,7 +561,7 @@ JSBool NativeJSWrapper::GetPropertyByName(jsval id, jsval *vp) {
   if (!ConvertNativeToJS(js_context_, return_value.v(), vp)) {
     RaiseException(js_context_,
                    "Failed to convert native property %s value(%s) to jsval",
-                   name.c_str(), return_value.v().Print().c_str());
+                   utf8_name.get(), return_value.v().Print().c_str());
     return JS_FALSE;
   }
   return JS_TRUE;
@@ -589,16 +577,11 @@ JSBool NativeJSWrapper::SetPropertyByName(jsval id, jsval js_val) {
   if (!idstr)
     return JS_FALSE;
 
-  AutoLocalRootScope local_root_scope(js_context_);
-  if (!local_root_scope.good())
-    return JS_FALSE;
-
   const jschar *utf16_name = JS_GetStringChars(idstr);
   size_t name_length = JS_GetStringLength(idstr);
-  std::string name;
-  ConvertStringUTF16ToUTF8(utf16_name, name_length, &name);
+  UTF16ToUTF8Converter utf8_name(utf16_name, name_length);
   Variant prototype;
-  if (scriptable_->GetPropertyInfo(name.c_str(), &prototype) ==
+  if (scriptable_->GetPropertyInfo(utf8_name.get(), &prototype) ==
       ScriptableInterface::PROPERTY_NOT_EXIST) {
     // This must be a dynamic property which is no more available.
     // Remove the property and fallback to the default handler.
@@ -613,14 +596,14 @@ JSBool NativeJSWrapper::SetPropertyByName(jsval id, jsval js_val) {
   if (!ConvertJSToNative(js_context_, this, prototype, js_val, &value)) {
     RaiseException(js_context_,
                    "Failed to convert JS property %s value(%s) to native.",
-                   name.c_str(), PrintJSValue(js_context_, js_val).c_str());
+                   utf8_name.get(), PrintJSValue(js_context_, js_val).c_str());
     return JS_FALSE;
   }
 
-  if (!scriptable_->SetProperty(name.c_str(), value)) {
+  if (!scriptable_->SetProperty(utf8_name.get(), value)) {
     RaiseException(js_context_,
                    "Failed to set native property %s (may be readonly).",
-                   name.c_str());
+                   utf8_name.get());
     FreeNativeValue(value);
     return JS_FALSE;
   }
@@ -633,6 +616,8 @@ class NameCollector {
   NameCollector(std::vector<std::string> *names) : names_(names) { }
   bool Collect(const char *name, ScriptableInterface::PropertyType type,
                const Variant &value) {
+    GGL_UNUSED(type);
+    GGL_UNUSED(value);
     names_->push_back(name);
     return true;
   }
@@ -693,18 +678,13 @@ JSBool NativeJSWrapper::ResolveProperty(jsval id, uintN flags,
   if (!JSVAL_IS_STRING(id))
     return JS_TRUE;
 
-  AutoLocalRootScope local_root_scope(js_context_);
-  if (!local_root_scope.good())
-    return JS_FALSE;
-
-  JSString *idstr = JS_ValueToString(js_context_, id);
+  JSString *idstr = JSVAL_TO_STRING(id);
   if (!idstr)
     return JS_FALSE;
 
   const jschar *utf16_name = JS_GetStringChars(idstr);
   size_t name_length = JS_GetStringLength(idstr);
-  std::string name;
-  ConvertStringUTF16ToUTF8(utf16_name, name_length, &name);
+  UTF16ToUTF8Converter utf8_name(utf16_name, name_length);
 
   // The JS program defines a new symbol. This has higher priority than the
   // properties of the global scriptable object.
@@ -713,14 +693,14 @@ JSBool NativeJSWrapper::ResolveProperty(jsval id, uintN flags,
 
   Variant prototype;
   ScriptableInterface::PropertyType type =
-      scriptable_->GetPropertyInfo(name.c_str(), &prototype);
+      scriptable_->GetPropertyInfo(utf8_name.get(), &prototype);
   if (type == ScriptableInterface::PROPERTY_NOT_EXIST) {
-    if (name == "toString") {
+    if (strcmp(utf8_name.get(), "toString") == 0) {
       // Define a default toString() operator to ease debugging.
       JS_DefineUCFunction(js_context_, js_object_, utf16_name, name_length,
                           WrapperDefaultToString, 0, 0);
       *objp = js_object_;
-    } else if (name == "__NATIVE_CLASS_ID__") {
+    } else if (strcmp(utf8_name.get(), "__NATIVE_CLASS_ID__") == 0) {
       // Register __NATIVE_CLASS_ID__ property for JS debugging.
       jsval js_val;
       ConvertNativeToJS(js_context_,
