@@ -21,14 +21,15 @@ var kPluginBoxHeight = 124;
 var kDefaultPluginRows = 3;
 var kDefaultPluginColumns = 4;
 var kCategoryGap = 15;
-var kFixedExtraWidth = 178;
+var kFixedExtraWidth = 142;
 var kFixedExtraHeight = 183;
 var kBorderMarginH = 12;
 var kBorderMarginV = 30;
+var kWindowMarginH = 12;
+var kCategoryItemWidth = 140;
+
 var kMinWidth = kFixedExtraWidth + 3 * kPluginBoxWidth + kBorderMarginH;
 var kMinHeight = kFixedExtraHeight + kPluginBoxHeight + kBorderMarginV;
-var kMaxWidth = kFixedExtraWidth + 5 * kPluginBoxWidth + kBorderMarginH;
-var kMaxHeight = kFixedExtraHeight + 4 * kPluginBoxHeight + kBorderMarginV;
 
 // Default layout: 714x555.
 var gPluginRows = kDefaultPluginRows;
@@ -53,10 +54,39 @@ function init() {
   UpdateLanguageBox();
 }
 
+function init_layout() {
+  var category_width = 0;
+  for (var i in kTopCategories) {
+    var category = kTopCategories[i];
+    category_item_ruler.value = GetDisplayCategory(category);
+    var width = category_item_ruler.idealBoundingRect.width;
+    category_width = Math.max(category_width, width);
+  }
+  for (var i in kBottomCategories) {
+    var category = kBottomCategories[i];
+    category_item_ruler.value = GetDisplayCategory(category);
+    var width = category_item_ruler.idealBoundingRect.width;
+    category_width = Math.max(category_width, width);
+  }
+  kCategoryItemWidth = category_width;
+  category_width = category_width + 20;
+  if (category_width > categories_div.offsetWidth) {
+    categories_div.width = category_width;
+    plugins_div.x = categories_div.offsetX + category_width + 10;
+    plugin_info_div.x = plugins_div.offsetX;
+    kFixedExtraWidth = category_width + kWindowMarginH + 10;
+    kMinWidth = kFixedExtraWidth + 3 * kPluginBoxWidth + kBorderMarginH;
+    debug.trace("category_width:" + category_width);
+    debug.trace("kFixedExtraWidth:" + kFixedExtraWidth);
+  }
+  view.removeElement(category_item_ruler);
+}
+
 function view_onopen() {
   var screen_width = system.screen.size.width;
   var screen_height = system.screen.size.height;
   var width, height;
+  init_layout();
   if (screen_width >= 1024) {
     width = kFixedExtraWidth + 4 * kPluginBoxWidth + kBorderMarginH;
   } else if (screen_width >= 800) {
@@ -92,13 +122,9 @@ function view_oncontextmenu() {
 function view_onsizing() {
   if (event.width < kMinWidth) {
     event.width = kMinWidth;
-//  } else if (event.width > kMaxWidth) {
-//    event.width = kMaxWidth;
   }
   if (event.height < kMinHeight) {
     event.height = kMinHeight;
-//  } else if (event.height > kMaxHeight) {
-//    event.height = kMaxHeight;
   }
 }
 
@@ -213,31 +239,30 @@ function GetDisplayCategory(category) {
 var gUpdatingLanguageBox = false;
 function UpdateLanguageBox() {
   gUpdatingLanguageBox = true;
-  var default_language = framework.system.languageCode().toLowerCase();
-  if (!default_language || !gPlugins[default_language] ||
-      !GetDisplayLanguage(default_language))
-    default_language = "en";
-  gadget.debug.trace("Default language: " + default_language);
 
   language_box.removeAllElements();
   var languages = [];
   for (var language in gPlugins) {
-    var disp_lang = GetDisplayLanguage(language);
-    if (disp_lang)
-      languages.push({lang: language, disp: disp_lang});
+    if (language != kAllLanguage) {
+      var disp_lang = GetDisplayLanguage(language);
+      if (disp_lang)
+        languages.push({lang: language, disp: disp_lang});
+    }
   }
   languages.sort(function(a, b) { return a.disp.localeCompare(b.disp); });
+  languages.unshift({
+    lang: kAllLanguage,
+    disp: GetDisplayLanguage(kAllLanguage)
+  });
   for (var i = 0; i < languages.length; i++) {
     var language = languages[i].lang;
     language_box.appendElement(
       "<item name='" + language +
       "'><label vAlign='middle' size='10'>" + languages[i].disp +
       "</label></item>");
-
-    if (default_language == language)
-      language_box.selectedIndex = language_box.children.count - 1;
   }
-  SelectLanguage(default_language);
+  language_box.selectedIndex = 0;
+  SelectLanguage(kAllLanguage);
   gUpdatingLanguageBox = false;
 }
 
@@ -248,11 +273,12 @@ function SelectLanguage(language) {
 
 function AddCategoryButton(category, y) {
   categories_div.appendElement(
-    "<label x='5%' width='90%' height='" + kCategoryButtonHeight + "' y='" + y +
+    "<label x='10' width='" + kCategoryItemWidth + "' height='" +
+    kCategoryButtonHeight + "' y='" + y +
     "' align='left' vAlign='middle' enabled='true' color='#FFFFFF' name='" +
     category + "' size='10' trimming='character-ellipsis'" +
     " onmouseover='category_onmouseover()' onmouseout='category_onmouseout()'" +
-    " onclick='SelectCategory(\"" + category + "\")'>&#160;" +
+    " onclick='SelectCategory(\"" + category + "\")'>" +
     GetDisplayCategory(category) + "</label>");
 }
 
@@ -394,16 +420,21 @@ function UpdateAddButtonVisualStatus(plugin) {
 
 function add_button_onmousedown(index) {
   // Reset the button status if the user clicks on it.
-  SetDownloadStatus(gCurrentPlugins[index], kDownloadStatusNone);
+  if (gCurrentCategory != kCategoryUpdates)
+    SetDownloadStatus(gCurrentPlugins[index], kDownloadStatusNone);
 }
 
 function add_button_onclick(index) {
   var plugin = gCurrentPlugins[index];
   if (plugin.download_status != kDownloadStatusAdding) {
+    var is_updating = (gCurrentCategory == kCategoryUpdates);
     plugin.button = event.srcElement;
     SetDownloadStatus(plugin, kDownloadStatusAdding);
     if (gadgetBrowserUtils.needDownloadGadget(plugin.id)) {
-      DownloadPlugin(plugin);
+      DownloadPlugin(plugin, is_updating);
+    } else if (is_updating) {
+      // The gadget has already been updated.
+      SetDownloadStatus(plugin, kDownloadStatusAdded);
     } else {
       if (AddPlugin(plugin) >= 0)
         SetDownloadStatus(plugin, kDownloadStatusAdded);
@@ -430,13 +461,15 @@ function plugin_title_onmouseout(index) {
 }
 
 function add_button_onmouseover(index) {
-  if (gCurrentPlugins[index].download_status != kDownloadStatusAdding)
+  if (gCurrentCategory != kCategoryUpdates &&
+      gCurrentPlugins[index].download_status != kDownloadStatusAdding)
     SetDownloadStatus(gCurrentPlugins[index], kDownloadStatusNone);
   MouseOverPlugin(event.srcElement.parentElement, index);
 }
 
 function add_button_onmouseout(index) {
-  if (gCurrentPlugins[index].download_status != kDownloadStatusAdding)
+  if (gCurrentCategory != kCategoryUpdates &&
+      gCurrentPlugins[index].download_status != kDownloadStatusAdding)
     SetDownloadStatus(gCurrentPlugins[index], kDownloadStatusNone);
   MouseOutPlugin(event.srcElement.parentElement, index);
 }
