@@ -47,12 +47,6 @@ static const double kMaximumZoom = 2.0;
 static const int kStopMoveDragTimeout = 200;
 static const char kMainViewWindowRole[] = "Google-Gadgets";
 
-// Minimal interval between queue draws.
-static const unsigned int kQueueDrawInterval = 40;
-
-// Maximum live duration of queue draw timer.
-static const uint64_t kQueueDrawTimerDuration = 1000;
-
 class SingleViewHost::Impl {
  public:
   Impl(SingleViewHost *owner, ViewHostInterface::Type type,
@@ -92,13 +86,9 @@ class SingleViewHost::Impl {
       is_keep_above_(false),
       move_dragging_(false),
       enable_signals_(true),
-      draw_queued_(false),
-      draw_finished_(false),
-      queue_draw_timer_(0),
-      last_queue_draw_time_(0),
       queue_resize_timer_(0),
-      last_allocated_width_(0),
-      last_allocated_height_(0),
+      fixed_width_from_view_(0),
+      fixed_height_from_view_(0),
       feedback_handler_(NULL),
       can_close_dialog_(false) {
     ASSERT(owner);
@@ -115,10 +105,6 @@ class SingleViewHost::Impl {
     // To make sure that it won't be accessed anymore.
     view_ = NULL;
 
-    if (queue_draw_timer_)
-      g_source_remove(queue_draw_timer_);
-    queue_draw_timer_ = 0;
-
     if (queue_resize_timer_)
       g_source_remove(queue_resize_timer_);
     queue_resize_timer_ = 0;
@@ -234,8 +220,8 @@ class SingleViewHost::Impl {
     g_signal_connect(G_OBJECT(fixed_), "size-allocate",
                      G_CALLBACK(FixedSizeAllocateHandler), this);
 
-    g_signal_connect(G_OBJECT(widget_), "expose-event",
-                     G_CALLBACK(ExposeHandler), this);
+    g_signal_connect(G_OBJECT(fixed_), "set-focus-child",
+                     G_CALLBACK(FixedSetFocusChildHandler), this);
 
     // For details and main view, the view is bound to the toplevel window
     // instead of the GtkFixed widget, to get better performance and make the
@@ -244,6 +230,7 @@ class SingleViewHost::Impl {
 
     gtk_widget_realize(fixed_);
     gtk_widget_realize(window_);
+    DLOG("Window created: %p, fixed: %p", window_, fixed_);
     on_view_changed_signal_();
   }
 
@@ -271,6 +258,11 @@ class SingleViewHost::Impl {
     int width = static_cast<int>(ceil(view_->GetWidth() * zoom));
     int height = static_cast<int>(ceil(view_->GetHeight() * zoom));
 
+    // Stores the expected size of the GtkFixed widget, which will be used in
+    // FixedSizeAllocateHandler().
+    fixed_width_from_view_ = width;
+    fixed_height_from_view_ = height;
+
     GtkRequisition req;
     gtk_widget_set_size_request(widget_, width, height);
     gtk_widget_size_request(window_, &req);
@@ -289,8 +281,6 @@ class SingleViewHost::Impl {
       win_width_ = req.width;
       win_height_ = req.height;
     }
-
-    DLOG("New window size: %d %d", req.width, req.height);
   }
 
   void QueueResize() {
@@ -307,33 +297,13 @@ class SingleViewHost::Impl {
       DLOG("SingleViewHost::EnableInputShapeMask(%s)",
            enable ? "true" : "false");
       binder_->EnableInputShapeMask(enable);
-      QueueDraw();
     }
   }
 
   void QueueDraw() {
     ASSERT(GTK_IS_WIDGET(widget_));
-    draw_finished_ = false;
-    if (queue_draw_timer_) {
-      draw_queued_ = true;
-      return;
-    }
-
-    uint64_t current_time = GetCurrentTime();
-    if (current_time - last_queue_draw_time_ >= kQueueDrawInterval) {
-      gtk_widget_queue_draw(widget_);
-      draw_queued_ = false;
-      last_queue_draw_time_ = current_time;
-    } else {
-      draw_queued_ = true;
-    }
-
-    // Can't call view's GetCaption() here, because at this point, view might
-    // not be fully initialized yet.
-    DLOG("Install queue draw timer of view: %p", view_);
-    queue_draw_timer_ = g_timeout_add(kQueueDrawInterval,
-                                      QueueDrawTimeoutHandler,
-                                      this);
+    if (binder_)
+      binder_->QueueDraw();
   }
 
   void SetResizable(ViewInterface::ResizableMode mode) {
@@ -363,6 +333,7 @@ class SingleViewHost::Impl {
   }
 
   void SetShowCaptionAlways(bool always) {
+    GGL_UNUSED(always);
     // SingleViewHost will always show caption when window decorator is shown.
   }
 
@@ -444,6 +415,9 @@ class SingleViewHost::Impl {
     // after the window is shown.
     LoadWindowStates();
 
+    // Make sure the view is inside screen.
+    EnsureInsideScreen();
+
     // Main view and details view doesn't support modal.
     if (type_ == ViewHostInterface::VIEW_HOST_OPTIONS && modal) {
       can_close_dialog_ = false;
@@ -726,9 +700,27 @@ class SingleViewHost::Impl {
     SetCursor(ViewInterface::CURSOR_DEFAULT);
   }
 
+  void EnsureInsideScreen() {
+    GdkScreen *screen = gtk_widget_get_screen(window_);
+    int screen_width = gdk_screen_get_width(screen);
+    int screen_height = gdk_screen_get_height(screen);
+    int win_center_x = win_x_ + win_width_ / 2;
+    int win_center_y = win_y_ + win_width_ / 2;
+
+    if (win_center_x < 0 || win_center_x >= screen_width ||
+        win_center_y < 0 || win_center_y >= screen_height) {
+      DLOG("View is out of screen: sw: %d, sh: %d, x: %d, y: %d",
+           screen_width, screen_height, win_center_x, win_center_y);
+      win_x_ = (screen_width - win_width_) / 2;
+      win_y_ = (screen_height - win_height_) / 2;
+      gtk_window_move(GTK_WINDOW(window_), win_x_, win_y_);
+    }
+  }
+
   // gtk signal handlers.
   static gboolean FocusInHandler(GtkWidget *widget, GdkEventFocus *event,
                                  gpointer user_data) {
+    GGL_UNUSED(event);
     DLOG("FocusInHandler(%p)", widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     if (impl->move_dragging_)
@@ -739,6 +731,8 @@ class SingleViewHost::Impl {
 #ifdef _DEBUG
   static gboolean FocusOutHandler(GtkWidget *widget, GdkEventFocus *event,
                                   gpointer user_data) {
+    GGL_UNUSED(event);
+    GGL_UNUSED(user_data);
     DLOG("FocusOutHandler(%p)", widget);
     return FALSE;
   }
@@ -754,6 +748,7 @@ class SingleViewHost::Impl {
   }
 
   static void WindowShowHandler(GtkWidget *widget, gpointer user_data) {
+    GGL_UNUSED(widget);
     DLOG("View window is going to be shown.");
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     if (impl->view_ && impl->enable_signals_)
@@ -761,6 +756,7 @@ class SingleViewHost::Impl {
   }
 
   static void WindowHideHandler(GtkWidget *widget, gpointer user_data) {
+    GGL_UNUSED(widget);
     DLOG("View window is going to be hidden.");
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     if (impl->view_ && impl->enable_signals_) {
@@ -780,6 +776,7 @@ class SingleViewHost::Impl {
 
   static gboolean ConfigureHandler(GtkWidget *widget, GdkEventConfigure *event,
                                    gpointer user_data) {
+    GGL_UNUSED(widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     if (impl->enable_signals_) {
       if (impl->win_x_ != event->x || impl->win_y_ != event->y) {
@@ -801,6 +798,7 @@ class SingleViewHost::Impl {
 
   static void DialogResponseHandler(GtkDialog *dialog, gint response,
                                     gpointer user_data) {
+    GGL_UNUSED(dialog);
     DLOG("%s button clicked in options dialog.",
          response == GTK_RESPONSE_OK ? "Ok" :
          response == GTK_RESPONSE_CANCEL ? "Cancel" : "No");
@@ -837,7 +835,7 @@ class SingleViewHost::Impl {
 #endif
       }
 
-      if (impl->draw_queued_ || !impl->draw_finished_)
+      if (impl->binder_ && impl->binder_->DrawQueued())
         return TRUE;
 
       int button = ConvertGdkModifierToButton(event->state);
@@ -857,7 +855,6 @@ class SingleViewHost::Impl {
           double view_width = new_width / impl->resize_view_zoom_;
           double view_height = new_height / impl->resize_view_zoom_;
           if (impl->view_->OnSizing(&view_width, &view_height)) {
-            DLOG("Resize view to: %lf %lf", view_width, view_height);
             impl->view_->SetSize(view_width, view_height);
             width = impl->view_->GetWidth() * impl->resize_view_zoom_;
             height = impl->view_->GetHeight() * impl->resize_view_zoom_;
@@ -867,7 +864,6 @@ class SingleViewHost::Impl {
           double yzoom = new_height / impl->resize_view_height_;
           double zoom = std::min(xzoom, yzoom);
           zoom = Clamp(zoom, kMinimumZoom, kMaximumZoom);
-          DLOG("Zoom view to: %lf", zoom);
           impl->view_->GetGraphics()->SetZoom(zoom);
           impl->view_->MarkRedraw();
           width = impl->resize_view_width_ * zoom;
@@ -886,9 +882,6 @@ class SingleViewHost::Impl {
           int win_width = impl->resize_win_width_ + int(delta_x);
           int win_height = impl->resize_win_height_ + int(delta_y);
           gdk_window_move_resize(widget->window, x, y, win_width, win_height);
-          gdk_window_process_updates(widget->window, TRUE);
-          DLOG("Move resize window: x:%d, y:%d, w:%d, h:%d", x, y,
-               win_width, win_height);
         }
 
         return TRUE;
@@ -901,6 +894,8 @@ class SingleViewHost::Impl {
 
   static gboolean ButtonReleaseHandler(GtkWidget *widget, GdkEventButton *event,
                                        gpointer user_data) {
+    GGL_UNUSED(widget);
+    GGL_UNUSED(event);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     if (impl->resize_width_mode_ || impl->resize_height_mode_) {
       impl->StopResizeDrag();
@@ -912,6 +907,7 @@ class SingleViewHost::Impl {
   static void WidgetSizeRequestHandler(GtkWidget *widget,
                                       GtkRequisition *requisition,
                                       gpointer user_data) {
+    GGL_UNUSED(widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     double zoom = impl->view_->GetGraphics()->GetZoom();
     if (impl->resizable_mode_ == ViewInterface::RESIZABLE_FALSE) {
@@ -940,16 +936,14 @@ class SingleViewHost::Impl {
   static void FixedSizeAllocateHandler(GtkWidget *widget,
                                        GtkAllocation *allocation,
                                        gpointer user_data) {
+    GGL_UNUSED(widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
-    DLOG("Size allocate(%d, %d)", allocation->width, allocation->height);
     if (GTK_WIDGET_VISIBLE(impl->window_) &&
         !impl->resize_width_mode_ && !impl->resize_height_mode_ &&
+        !impl->queue_resize_timer_ &&
         allocation->width >= 1 && allocation->height >= 1 &&
-        (impl->last_allocated_width_ != allocation->width ||
-         impl->last_allocated_height_ != allocation->height)) {
-      impl->last_allocated_width_ = allocation->width;
-      impl->last_allocated_height_ = allocation->height;
-
+        (impl->fixed_width_from_view_ != allocation->width ||
+         impl->fixed_height_from_view_ != allocation->height)) {
       double old_width = impl->view_->GetWidth();
       double old_height = impl->view_->GetHeight();
       double old_zoom = impl->view_->GetGraphics()->GetZoom();
@@ -959,7 +953,6 @@ class SingleViewHost::Impl {
         double new_height = allocation->height / old_zoom;
         if (impl->view_->OnSizing(&new_width, &new_height) &&
             (new_width != old_width || new_height != old_height)) {
-          DLOG("Resize view to: %lf %lf", new_width, new_height);
           impl->view_->SetSize(new_width, new_height);
         }
       } else if (impl->resizable_mode_ == ViewInterface::RESIZABLE_ZOOM &&
@@ -969,7 +962,6 @@ class SingleViewHost::Impl {
         double new_zoom = Clamp(std::min(xzoom, yzoom),
                                 kMinimumZoom, kMaximumZoom);
         if (old_zoom != new_zoom) {
-          DLOG("Zoom view to: %lf", new_zoom);
           impl->view_->GetGraphics()->SetZoom(new_zoom);
           impl->view_->MarkRedraw();
         }
@@ -996,25 +988,6 @@ class SingleViewHost::Impl {
     return FALSE;
   }
 
-  static gboolean QueueDrawTimeoutHandler(gpointer data) {
-    Impl *impl = reinterpret_cast<Impl *>(data);
-    uint64_t current_time = GetCurrentTime();
-    if (impl->draw_queued_) {
-      ASSERT(GTK_IS_WIDGET(impl->widget_));
-      gtk_widget_queue_draw(impl->widget_);
-      impl->draw_queued_ = false;
-      impl->last_queue_draw_time_ = current_time;
-    }
-
-    if (current_time - impl->last_queue_draw_time_ > kQueueDrawTimerDuration) {
-      DLOG("Remove queue draw timer of view: %p (%s)", impl->view_,
-           impl->view_->GetCaption().c_str());
-      impl->queue_draw_timer_ = 0;
-      return FALSE;
-    }
-    return TRUE;
-  }
-
   static gboolean QueueResizeTimeoutHandler(gpointer data) {
     Impl *impl = reinterpret_cast<Impl *>(data);
     impl->AdjustWindowSize();
@@ -1022,12 +995,30 @@ class SingleViewHost::Impl {
     return false;
   }
 
-  static gboolean ExposeHandler(GtkWidget *widget, GdkEventExpose *event,
-                                gpointer data) {
-    Impl *impl = reinterpret_cast<Impl *>(data);
-    impl->draw_finished_ = true;
-    // Make sure view widget binder can receive this event.
-    return FALSE;
+  // Some elements may create gtk native widgets under this widget. When such
+  // a widget get focus, we must update the focus chain though the view
+  // hierachy.
+  static void FixedSetFocusChildHandler(GtkContainer *container,
+                                        GtkWidget *widget,
+                                        gpointer user_data) {
+    Impl *impl = reinterpret_cast<Impl *>(user_data);
+    DLOG("FixedSetFocusChildHandler widget: %p, view: %p, child: %p",
+         container, impl->view_, widget);
+    if (widget) {
+      // Send fake MOUSE_DOWN/MOUSE_UP events to update the focus chain
+      // from the hosted view down to the element containing the native
+      // widget.
+      gint x = widget->allocation.x + widget->allocation.width / 2;
+      gint y = widget->allocation.y + widget->allocation.height / 2;
+      double view_x = 0, view_y = 0;
+      impl->NativeWidgetCoordToViewCoord(x, y, &view_x, &view_y);
+      MouseEvent down_event(Event::EVENT_MOUSE_DOWN, view_x, view_y,
+                            0, 0, MouseEvent::BUTTON_LEFT, 0);
+      impl->view_->OnMouseEvent(down_event);
+      MouseEvent up_event(Event::EVENT_MOUSE_UP, view_x, view_y,
+                          0, 0, MouseEvent::BUTTON_LEFT, 0);
+      impl->view_->OnMouseEvent(up_event);
+    }
   }
 
   SingleViewHost *owner_;
@@ -1083,14 +1074,9 @@ class SingleViewHost::Impl {
   bool move_dragging_;
   bool enable_signals_;
 
-  bool draw_queued_;
-  bool draw_finished_;
-  guint queue_draw_timer_;
-  uint64_t last_queue_draw_time_;
-
   guint queue_resize_timer_;
-  gint last_allocated_width_;
-  gint last_allocated_height_;
+  int fixed_width_from_view_;
+  int fixed_height_from_view_;
 
   Slot1<bool, int> *feedback_handler_;
   bool can_close_dialog_; // Only useful when a model dialog is running.
