@@ -18,6 +18,7 @@
 #include <string>
 #include <vector>
 #include <cairo.h>
+#include <gdk/gdk.h>
 
 #include "view_widget_binder.h"
 #include <ggadget/common.h>
@@ -28,6 +29,8 @@
 #include <ggadget/view_host_interface.h>
 #include <ggadget/string_utils.h>
 #include <ggadget/small_object.h>
+#include <ggadget/clip_region.h>
+#include <ggadget/math_utils.h>
 #include "cairo_canvas.h"
 #include "cairo_graphics.h"
 #include "key_convert.h"
@@ -52,6 +55,12 @@ static const double kDragThreshold = 3;
 static const uint64_t kFPSCountDuration = 5000;
 #endif
 
+// Update input shape mask once per second.
+static const uint64_t kUpdateMaskInterval = 1000;
+
+// Minimal interval between self draws.
+static const unsigned int kSelfDrawInterval = 40;
+
 class ViewWidgetBinder::Impl : public SmallObject<> {
  public:
   Impl(ViewInterface *view,
@@ -60,13 +69,18 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
     : view_(view),
       host_(host),
       widget_(widget),
+#if GTK_CHECK_VERSION(2,10,0)
+      input_shape_mask_(NULL),
+      last_mask_time_(0),
+      should_update_input_shape_mask_(false),
+      enable_input_shape_mask_(false),
+#endif
       handlers_(new gulong[kEventHandlersNum]),
       current_drag_event_(NULL),
       on_zoom_connection_(NULL),
       dbl_click_(false),
       composited_(false),
       no_background_(no_background),
-      enable_input_shape_mask_(false),
       focused_(false),
       button_pressed_(false),
 #ifdef GRAB_POINTER_EXPLICITLY
@@ -79,7 +93,11 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
       zoom_(1.0),
       mouse_down_x_(-1),
       mouse_down_y_(-1),
-      mouse_down_hittest_(ViewInterface::HT_CLIENT) {
+      mouse_down_hittest_(ViewInterface::HT_CLIENT),
+      self_draw_(false),
+      self_draw_timer_(0),
+      last_self_draw_time_(0),
+      sys_clip_region_(NULL) {
     ASSERT(view);
     ASSERT(host);
     ASSERT(GTK_IS_WIDGET(widget));
@@ -87,6 +105,8 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
     g_object_ref(G_OBJECT(widget_));
     gtk_widget_set_app_paintable(widget_, TRUE);
+    gtk_widget_set_double_buffered(widget_, FALSE);
+
     gint events = GDK_EXPOSURE_MASK |
                   GDK_FOCUS_CHANGE_MASK |
                   GDK_ENTER_NOTIFY_MASK |
@@ -131,6 +151,16 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
   ~Impl() {
     view_ = NULL;
 
+    if (self_draw_timer_) {
+      g_source_remove(self_draw_timer_);
+      self_draw_timer_ = 0;
+    }
+
+    if (sys_clip_region_) {
+      gdk_region_destroy(sys_clip_region_);
+      sys_clip_region_ = NULL;
+    }
+
     for (size_t i = 0; i < kEventHandlersNum; ++i) {
       if (handlers_[i] > 0)
         g_signal_handler_disconnect(G_OBJECT(widget_), handlers_[i]);
@@ -152,6 +182,11 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
     }
 
     g_object_unref(G_OBJECT(widget_));
+#if GTK_CHECK_VERSION(2,10,0)
+    if (input_shape_mask_) {
+      g_object_unref(G_OBJECT(input_shape_mask_));
+    }
+#endif
   }
 
   void OnZoom(double zoom) {
@@ -166,12 +201,156 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
     }
   }
 
+  GdkRegion *CreateExposeRegionFromViewClipRegion() {
+    GdkRegion *region = gdk_region_new();
+    const ClipRegion *view_region = view_->GetClipRegion();
+    size_t count = view_region->GetRectangleCount();
+    if (count) {
+      Rectangle rect;
+      GdkRectangle gdk_rect;
+      for (size_t i = 0; i < count; ++i) {
+        rect = view_region->GetRectangle(i);
+        if (zoom_ != 1.0) {
+          rect.Zoom(zoom_);
+          rect.Integerize(true);
+        }
+        gdk_rect.x = static_cast<int>(rect.x);
+        gdk_rect.y = static_cast<int>(rect.y);
+        gdk_rect.width = static_cast<int>(rect.w);
+        gdk_rect.height = static_cast<int>(rect.h);
+        gdk_region_union_with_rect(region, &gdk_rect);
+      }
+    }
+    return region;
+  }
+
+  void AddGdkRectToSystemClipRegion(GdkRectangle *rect) {
+    if (!sys_clip_region_)
+      sys_clip_region_ = gdk_region_new();
+    gdk_region_union_with_rect(sys_clip_region_, rect);
+  }
+
+  void AddGdkRegionToSystemClipRegion(GdkRegion *region) {
+    if (!sys_clip_region_)
+      sys_clip_region_ = gdk_region_new();
+    gdk_region_union(sys_clip_region_, region);
+  }
+
+  void AddWindowUpdateAreaToSystemClipRegion() {
+    GdkRegion *region = gdk_window_get_update_area(widget_->window);
+    if (region) {
+      if (!sys_clip_region_) {
+        sys_clip_region_ = region;
+      } else {
+        gdk_region_union(sys_clip_region_, region);
+        gdk_region_destroy(region);
+      }
+    }
+  }
+
+  void AddGdkRectToViewClipRegion(const GdkRectangle &gdk_rect) {
+    Rectangle rect(gdk_rect.x, gdk_rect.y, gdk_rect.width, gdk_rect.height);
+    rect.Zoom(1.0 / zoom_);
+    rect.Integerize(true);
+    view_->AddRectangleToClipRegion(rect);
+  }
+
+  void AddGdkRegionToViewClipRegion(GdkRegion *region) {
+    if (!gdk_region_empty(region)) {
+      GdkRectangle *rects;
+      gint n_rects;
+      gdk_region_get_rectangles(region, &rects, &n_rects);
+      for (gint i = 0; i < n_rects; ++i) {
+        AddGdkRectToViewClipRegion(rects[i]);
+      }
+      g_free(rects);
+    }
+  }
+
+#if GTK_CHECK_VERSION(2,10,0)
+  bool ShouldUpdateInputShapeMask() {
+    gint width, height;
+    gdk_drawable_get_size(widget_->window, &width, &height);
+
+    bool update_input_shape_mask = enable_input_shape_mask_ &&
+        (GetCurrentTime() - last_mask_time_ > kUpdateMaskInterval) &&
+        no_background_ && composited_;
+
+    // We need set input shape mask if there is no background.
+    if (update_input_shape_mask) {
+      if (input_shape_mask_) {
+        gint mask_width, mask_height;
+        gdk_drawable_get_size(GDK_DRAWABLE(input_shape_mask_),
+                              &mask_width, &mask_height);
+        if (mask_width != width || mask_height != height) {
+          // input shape mask needs recreate.
+          g_object_unref(G_OBJECT(input_shape_mask_));
+          input_shape_mask_ = NULL;
+        }
+      }
+
+      if (input_shape_mask_ == NULL) {
+        GdkRectangle rect;
+        rect.x = 0;
+        rect.y = 0;
+        rect.width = width;
+        rect.height = height;
+        input_shape_mask_ = gdk_pixmap_new(NULL, width, height, 1);
+
+        // Redraw whole view.
+        AddGdkRectToSystemClipRegion(&rect);
+      }
+    }
+
+    return update_input_shape_mask;
+  }
+#endif
+
+  GdkRegion *GetInvalidateRegion() {
+    view_->Layout();
+    AddWindowUpdateAreaToSystemClipRegion();
+#if GTK_CHECK_VERSION(2,10,0)
+    should_update_input_shape_mask_ = ShouldUpdateInputShapeMask();
+#endif
+    GdkRegion *region = CreateExposeRegionFromViewClipRegion();
+    if (sys_clip_region_) {
+      AddGdkRegionToViewClipRegion(sys_clip_region_);
+      gdk_region_union(region, sys_clip_region_);
+      gdk_region_destroy(sys_clip_region_);
+      sys_clip_region_ = NULL;
+    }
+    return region;
+  }
+
+  void SelfDraw() {
+    if (!widget_->window || !gdk_window_is_visible(widget_->window))
+      return;
+
+    self_draw_ = true;
+    GdkRegion *region = GetInvalidateRegion();
+    if (!gdk_region_empty(region)) {
+      gdk_window_invalidate_region(widget_->window, region, TRUE);
+      gdk_window_process_updates(widget_->window, TRUE);
+    }
+    gdk_region_destroy(region);
+    last_self_draw_time_ = GetCurrentTime();
+    self_draw_ = false;
+  }
+
   static gboolean ButtonPressHandler(GtkWidget *widget, GdkEventButton *event,
                                      gpointer user_data) {
-    DLOG("ButtonPressHandler.");
     Impl *impl = reinterpret_cast<Impl *>(user_data);
+    DLOG("ButtonPressHandler: widget: %p, view: %p, focused: %d, child: %p",
+         widget, impl->view_, impl->focused_,
+         gtk_window_get_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget))));
+
     EventResult result = EVENT_RESULT_UNHANDLED;
 
+    // Clicking on this widget removes any native child widget focus.
+    if (GTK_IS_WINDOW(widget)) {
+      gtk_window_set_focus(GTK_WINDOW(widget), NULL);
+    }
+
     impl->button_pressed_ = true;
     impl->host_->ShowTooltip("");
 
@@ -180,8 +359,10 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
       SimpleEvent e(Event::EVENT_FOCUS_IN);
       // Ignore the result.
       impl->view_->OnOtherEvent(e);
-      if (!gtk_widget_is_focus(widget))
+      if (!gtk_widget_is_focus(widget)) {
+        gtk_widget_grab_focus(widget);
         gdk_window_focus(impl->widget_->window, event->time);
+      }
     }
 
     int mod = ConvertGdkModifierToModifier(event->state);
@@ -229,6 +410,7 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean ButtonReleaseHandler(GtkWidget *widget, GdkEventButton *event,
                                        gpointer user_data) {
+    GGL_UNUSED(widget);
     DLOG("ButtonReleaseHandler.");
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     EventResult result = EVENT_RESULT_UNHANDLED;
@@ -276,6 +458,7 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean KeyPressHandler(GtkWidget *widget, GdkEventKey *event,
                                   gpointer user_data) {
+    GGL_UNUSED(widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     EventResult result = EVENT_RESULT_UNHANDLED;
     EventResult result2 = EVENT_RESULT_UNHANDLED;
@@ -320,6 +503,7 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean KeyReleaseHandler(GtkWidget *widget, GdkEventKey *event,
                                     gpointer user_data) {
+    GGL_UNUSED(widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     EventResult result = EVENT_RESULT_UNHANDLED;
 
@@ -338,12 +522,28 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
   static gboolean ExposeHandler(GtkWidget *widget, GdkEventExpose *event,
                                 gpointer user_data) {
     Impl *impl = reinterpret_cast<Impl *>(user_data);
-    gint width, height;
-    gdk_drawable_get_size(widget->window, &width, &height);
+
+    if (!impl->self_draw_) {
+      impl->AddGdkRegionToSystemClipRegion(event->region);
+      GdkRegion *invalidate_region = impl->GetInvalidateRegion();
+
+      // We can't update the region outside event->region, so update them in a
+      // new self draw request.
+      gdk_region_subtract(invalidate_region, event->region);
+      if (!gdk_region_empty(invalidate_region)) {
+        impl->AddGdkRegionToSystemClipRegion(invalidate_region);
+        if (!impl->self_draw_timer_) {
+          impl->self_draw_timer_ =
+              g_idle_add_full(GDK_PRIORITY_REDRAW, Impl::SelfDrawHandler,
+                              impl, NULL);
+        }
+      }
+      gdk_region_destroy(invalidate_region);
+    }
+
+    gdk_window_begin_paint_region(widget->window, event->region);
 
     cairo_t *cr = gdk_cairo_create(widget->window);
-    gdk_cairo_region(cr, event->region);
-    cairo_clip(cr);
 
     // If background is disabled, and if composited is enabled,  the window
     // needs clearing every times.
@@ -357,36 +557,36 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
     // Let View draw on the gdk window directly.
     // It's ok, because the View has canvas cache internally.
     CairoCanvas *canvas = new CairoCanvas(cr,
-                                          impl->view_->GetGraphics()->GetZoom(),
+                                          impl->zoom_,
                                           impl->view_->GetWidth(),
                                           impl->view_->GetHeight());
     ASSERT(canvas);
 
     impl->view_->Draw(canvas);
 
+    canvas->Destroy();
+    cairo_destroy(cr);
+
 #if GTK_CHECK_VERSION(2,10,0)
     // We need set input shape mask if there is no background.
-    if (impl->no_background_ && impl->composited_ &&
-        impl->enable_input_shape_mask_) {
-      // create an identical bitmap to use as shape mask
-      GdkBitmap *bitmap =
-        static_cast<GdkBitmap *>(gdk_pixmap_new(NULL, width, height, 1));
-      cairo_t *mask = gdk_cairo_create(bitmap);
-      CairoCanvas *mask_canvas =
-          new CairoCanvas(mask, impl->view_->GetGraphics()->GetZoom(),
-                          impl->view_->GetWidth(), impl->view_->GetHeight());
-      mask_canvas->ClearCanvas();
-      impl->view_->Draw(mask_canvas);
-      mask_canvas->Destroy();
-      cairo_destroy(mask);
-
-      gtk_widget_input_shape_combine_mask(widget, bitmap, 0, 0);
-      gdk_bitmap_unref(bitmap);
+    if (impl->should_update_input_shape_mask_ && impl->input_shape_mask_) {
+      cairo_t *mask_cr = gdk_cairo_create(impl->input_shape_mask_);
+      gdk_cairo_region(mask_cr, event->region);
+      cairo_clip(mask_cr);
+      cairo_set_operator(mask_cr, CAIRO_OPERATOR_CLEAR);
+      cairo_paint(mask_cr);
+      cairo_set_operator(mask_cr, CAIRO_OPERATOR_SOURCE);
+      gdk_cairo_set_source_pixmap(mask_cr, widget->window, 0, 0);
+      cairo_paint(mask_cr);
+      cairo_destroy(mask_cr);
+      gdk_window_input_shape_combine_mask(widget->window,
+                                          impl->input_shape_mask_, 0, 0);
+      impl->last_mask_time_ = GetCurrentTime();
     }
 #endif
 
-    canvas->Destroy();
-    cairo_destroy(cr);
+    // Copy off-screen buffer to screen.
+    gdk_window_end_paint(widget->window);
 
 #ifdef _DEBUG
     ++impl->draw_count_;
@@ -400,6 +600,7 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
       impl->draw_count_ = 0;
     }
 #endif
+
     return TRUE;
   }
 
@@ -491,6 +692,7 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean ScrollHandler(GtkWidget *widget, GdkEventScroll *event,
                                 gpointer user_data) {
+    GGL_UNUSED(widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     int delta_x = 0, delta_y = 0;
     if (event->direction == GDK_SCROLL_UP) {
@@ -513,6 +715,7 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean LeaveNotifyHandler(GtkWidget *widget, GdkEventCrossing *event,
                                      gpointer user_data) {
+    GGL_UNUSED(widget);
     if (event->mode != GDK_CROSSING_NORMAL ||
         event->detail == GDK_NOTIFY_INFERIOR)
       return FALSE;
@@ -534,6 +737,7 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean EnterNotifyHandler(GtkWidget *widget, GdkEventCrossing *event,
                                      gpointer user_data) {
+    GGL_UNUSED(widget);
     if (event->mode != GDK_CROSSING_NORMAL ||
         event->detail == GDK_NOTIFY_INFERIOR)
       return FALSE;
@@ -550,7 +754,11 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean FocusInHandler(GtkWidget *widget, GdkEventFocus *event,
                                  gpointer user_data) {
+    GGL_UNUSED(event);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
+    DLOG("FocusInHandler: widget: %p, view: %p, focused: %d, child: %p",
+         widget, impl->view_, impl->focused_,
+         gtk_window_get_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget))));
     if (!impl->focused_) {
       impl->focused_ = true;
       SimpleEvent e(Event::EVENT_FOCUS_IN);
@@ -561,7 +769,11 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static gboolean FocusOutHandler(GtkWidget *widget, GdkEventFocus *event,
                                   gpointer user_data) {
+    GGL_UNUSED(event);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
+    DLOG("FocusOutHandler: widget: %p, view: %p, focused: %d, child: %p",
+         widget, impl->view_, impl->focused_,
+         gtk_window_get_focus(GTK_WINDOW(gtk_widget_get_toplevel(widget))));
     if (impl->focused_) {
       impl->focused_ = false;
       SimpleEvent e(Event::EVENT_FOCUS_OUT);
@@ -599,6 +811,9 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 #ifdef GRAB_POINTER_EXPLICITLY
   static gboolean GrabBrokenHandler(GtkWidget *widget, GdkEvent *event,
                                     gpointer user_data) {
+    GGL_UNUSED(event);
+    GGL_UNUSED(widget);
+    GGL_UNUSED(user_data);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     impl->pointer_grabbed_ = false;
     return FALSE;
@@ -611,6 +826,8 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
                                       GtkSelectionData *data,
                                       guint info, guint time,
                                       gpointer user_data) {
+    GGL_UNUSED(widget);
+    GGL_UNUSED(info);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     if (!impl->current_drag_event_) {
       // There are some cases that multiple drag events are fired in one event
@@ -723,11 +940,14 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
 
   static void ScreenChangedHandler(GtkWidget *widget, GdkScreen *last_screen,
                                    gpointer user_data) {
+    GGL_UNUSED(widget);
+    GGL_UNUSED(last_screen);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     impl->SetupBackgroundMode();
   }
 
   static void CompositedChangedHandler(GtkWidget *widget, gpointer user_data) {
+    GGL_UNUSED(widget);
     Impl *impl = reinterpret_cast<Impl *>(user_data);
     impl->SetupBackgroundMode();
   }
@@ -755,16 +975,28 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
     return FALSE;
   }
 
+  static gboolean SelfDrawHandler(gpointer user_data) {
+    Impl *impl = reinterpret_cast<Impl *>(user_data);
+    impl->self_draw_timer_ = 0;
+    impl->SelfDraw();
+    return FALSE;
+  }
+
   ViewInterface *view_;
   ViewHostInterface *host_;
   GtkWidget *widget_;
+#if GTK_CHECK_VERSION(2,10,0)
+  GdkBitmap *input_shape_mask_;
+  uint64_t last_mask_time_;
+  bool should_update_input_shape_mask_;
+  bool enable_input_shape_mask_;
+#endif
   gulong *handlers_;
   DragEvent *current_drag_event_;
   Connection *on_zoom_connection_;
   bool dbl_click_;
   bool composited_;
   bool no_background_;
-  bool enable_input_shape_mask_;
   bool focused_;
   bool button_pressed_;
 #ifdef GRAB_POINTER_EXPLICITLY
@@ -779,6 +1011,11 @@ class ViewWidgetBinder::Impl : public SmallObject<> {
   double mouse_down_y_;
   ViewInterface::HitTest mouse_down_hittest_;
 
+  bool self_draw_;
+  guint self_draw_timer_;
+  uint64_t last_self_draw_time_;
+  GdkRegion *sys_clip_region_;
+
   struct EventHandlerInfo {
     const char *event;
     void (*handler)(void);
@@ -824,10 +1061,21 @@ ViewWidgetBinder::ViewWidgetBinder(ViewInterface *view,
 }
 
 void ViewWidgetBinder::EnableInputShapeMask(bool enable) {
-  impl_->enable_input_shape_mask_ = enable;
 #if GTK_CHECK_VERSION(2,10,0)
-  if (impl_->widget_ && impl_->no_background_ && impl_->composited_ && !enable)
-    gtk_widget_input_shape_combine_mask(impl_->widget_, NULL, 0, 0);
+  if (impl_->enable_input_shape_mask_ != enable) {
+    impl_->enable_input_shape_mask_ = enable;
+    if (impl_->widget_ && impl_->no_background_ &&
+        impl_->composited_ && !enable) {
+      if (impl_->widget_->window) {
+        gdk_window_input_shape_combine_mask(impl_->widget_->window, NULL, 0, 0);
+      }
+      if (impl_->input_shape_mask_) {
+        g_object_unref(G_OBJECT(impl_->input_shape_mask_));
+        impl_->input_shape_mask_ = NULL;
+      }
+    }
+    gtk_widget_queue_draw(impl_->widget_);
+  }
 #endif
 }
 
@@ -836,5 +1084,34 @@ ViewWidgetBinder::~ViewWidgetBinder() {
   impl_ = NULL;
 }
 
+void ViewWidgetBinder::QueueDraw() {
+  if (!impl_->self_draw_timer_) {
+    uint64_t current_time = GetCurrentTime();
+    if (current_time - impl_->last_self_draw_time_ >= kSelfDrawInterval) {
+      impl_->self_draw_timer_ =
+          g_idle_add_full(GDK_PRIORITY_REDRAW, Impl::SelfDrawHandler,
+                          impl_, NULL);
+    } else {
+      impl_->self_draw_timer_ =
+          g_timeout_add(kSelfDrawInterval -
+                        (current_time - impl_->last_self_draw_time_),
+                        Impl::SelfDrawHandler, impl_);
+    }
+  }
+}
+
+void ViewWidgetBinder::DrawImmediately() {
+  // Remove pending queue draw, as we don't need it anymore.
+  if (impl_->self_draw_timer_) {
+    g_source_remove(impl_->self_draw_timer_);
+    impl_->self_draw_timer_ = 0;
+  }
+  impl_->SelfDraw();
+}
+
+bool ViewWidgetBinder::DrawQueued() {
+  return impl_->self_draw_timer_ != 0;
+}
+
 } // namespace gtk
 } // namespace ggadget
