@@ -57,6 +57,7 @@ class ListBoxElement::Impl : public SmallObject<> {
     item_height_relative_(false),
     multiselect_(false),
     item_separator_(false) {
+    GGL_UNUSED(view);
   }
 
   ~Impl() {
@@ -231,7 +232,6 @@ class ListBoxElement::Impl : public SmallObject<> {
     if (item && !item->IsSelected()) {
       item->SetSelected(true);
       pending_scroll_ = 2;
-      owner_->QueueDraw();
       changed = true;
     }
 
@@ -332,6 +332,7 @@ ListBoxElement::ListBoxElement(View *view,
     : DivElement(view, tag_name, name),
       impl_(new Impl(this, view)) {
   SetEnabled(true);
+  SetXScrollable(false);
 }
 
 void ListBoxElement::DoClassRegister() {
