@@ -325,12 +325,6 @@ class  MainViewDecoratorBase::Impl : public SmallObject<> {
       gadget->ShowAboutDialog();
   }
 
-  void FeedbackMenuCallback(const char *) {
-    Gadget *gadget = owner_->GetGadget();
-    if (gadget)
-      gadget->OpenFeedbackURL();
-  }
-
   void RemoveMenuCallback(const char *) {
     Gadget *gadget = owner_->GetGadget();
     if (gadget)
@@ -785,6 +779,8 @@ void MainViewDecoratorBase::GetClientExtents(double *width,
 }
 
 bool MainViewDecoratorBase::OnClientSizing(double *width, double *height) {
+  GGL_UNUSED(width);
+  GGL_UNUSED(height);
   if (impl_->minimized_)
     *height = kVDMainMinimizedHeight;
   return true;
@@ -814,12 +810,6 @@ void MainViewDecoratorBase::OnAddDecoratorMenuItems(MenuInterface *menu) {
                   NewSlot(impl_, &Impl::AboutMenuCallback),
                   MenuInterface::MENU_ITEM_PRI_GADGET);
 
-    if (gadget->HasFeedbackURL()) {
-      menu->AddItem(GM_("MENU_ITEM_FEEDBACK"), 0, 0,
-                    NewSlot(impl_, &Impl::FeedbackMenuCallback),
-                    MenuInterface::MENU_ITEM_PRI_GADGET);
-    }
-
     // Use MENU_ITEM_PRI_GADGET to make sure that it's the last menu item.
     menu->AddItem(GM_("MENU_ITEM_REMOVE"), 0,
                   MenuInterface::MENU_ITEM_ICON_DELETE,
