@@ -25,6 +25,7 @@
 #include <ggadget/basic_element.h>
 #include <ggadget/gadget.h>
 #include <ggadget/gadget_consts.h>
+#include <ggadget/graphics_interface.h>
 #include <ggadget/main_loop_interface.h>
 #include <ggadget/math_utils.h>
 #include <ggadget/permissions.h>
@@ -109,7 +110,7 @@ class Plugin::Impl : public SmallObject<> {
         std::string content;
         if (ReadFileContents(url_or_file_.c_str(), &content) &&
             NewStream(content.size()) &&
-            WriteStream(content) == content.size()) {
+            WriteStream(content.c_str(), content.size()) == content.size()) {
           Done(NPRES_DONE);
         } else {
           Done(NPRES_NETWORK_ERR);
@@ -150,11 +151,10 @@ class Plugin::Impl : public SmallObject<> {
       return err == NPERR_NO_ERROR && stream_type == NP_NORMAL;
     }
 
-    size_t WriteStream(const std::string &data) {
+    size_t WriteStream(const void *data, size_t size) {
       ASSERT(stream_);
       size_t data_offset = 0;
       int idle_count = 0;
-      size_t size = data.size();
       while (data_offset < size) {
         int32 len = plugin_funcs_->writeready(&owner_->instance_, stream_);
         if (len <= 0) {
@@ -171,7 +171,7 @@ class Plugin::Impl : public SmallObject<> {
             &owner_->instance_, stream_,
             static_cast<int32>(stream_offset_), len,
             // This gets a non-const pointer.
-            const_cast<char *>(data.c_str()));
+            const_cast<void *>(data));
         // Error occurs.
         if (consumed < 0)
           return data_offset;
@@ -204,7 +204,7 @@ class Plugin::Impl : public SmallObject<> {
       return result;
     }
 
-    size_t OnDataReceived(const std::string &data) {
+    size_t OnDataReceived(const void *data, size_t size) {
       unsigned short status = 0;
       http_request_->GetStatus(&status);
       if (status == 200) {
@@ -232,13 +232,13 @@ class Plugin::Impl : public SmallObject<> {
           }
         }
 
-        size_t written = WriteStream(data);
-        if (written != data.size()) {
+        size_t written = WriteStream(data, size);
+        if (written != size) {
           Done(NPRES_NETWORK_ERR);
           return 0;
         }
       }
-      return data.size();
+      return size;
     }
 
     void OnStateChange() {
@@ -466,10 +466,18 @@ class Plugin::Impl : public SmallObject<> {
       Rectangle rect(invalid_rect->left, invalid_rect->top,
                      invalid_rect->right - invalid_rect->left,
                      invalid_rect->bottom - invalid_rect->top);
-      // The current plugins seems always invalid the whole rect.
-      // Otherwise we need to consider the zoom factor of the view.
-      element_->QueueDrawRect(rect);
       dirty_rect_.Union(rect);
+
+      // QueueDrawRect must be in the element's original coordinates, so zoom
+      // must be considered.
+      double zoom = element_->GetView()->GetGraphics()->GetZoom();
+      if (zoom != 1.0) {
+        rect.x /= zoom;
+        rect.y /= zoom;
+        rect.w /= zoom;
+        rect.h /= zoom;
+      }
+      element_->QueueDrawRect(rect);
     }
   }
 
@@ -547,12 +555,18 @@ class Plugin::Impl : public SmallObject<> {
   }
 
   static NPError NPN_RequestRead(NPStream *stream, NPByteRange *rangeList) {
+    GGL_UNUSED(stream);
+    GGL_UNUSED(rangeList);
     NOT_IMPLEMENTED();
     return NPERR_GENERIC_ERROR;
   }
 
   static NPError NPN_NewStream(NPP instance, NPMIMEType type,
                                const char *target, NPStream **stream) {
+    GGL_UNUSED(instance);
+    GGL_UNUSED(type);
+    GGL_UNUSED(target);
+    GGL_UNUSED(stream);
     // Plugin-produced stream is not supported.
     NOT_IMPLEMENTED();
     return NPERR_GENERIC_ERROR;
@@ -560,12 +574,19 @@ class Plugin::Impl : public SmallObject<> {
 
   static int32 NPN_Write(NPP instance, NPStream *stream,
                          int32 len, void *buffer) {
+    GGL_UNUSED(instance);
+    GGL_UNUSED(stream);
+    GGL_UNUSED(len);
+    GGL_UNUSED(buffer);
     NOT_IMPLEMENTED();
     return -1;
   }
 
   static NPError NPN_DestroyStream(NPP instance, NPStream *stream,
                                    NPReason reason) {
+    GGL_UNUSED(instance);
+    GGL_UNUSED(stream);
+    GGL_UNUSED(reason);
     NOT_IMPLEMENTED();
     return NPERR_GENERIC_ERROR;
   }
@@ -579,11 +600,13 @@ class Plugin::Impl : public SmallObject<> {
   }
 
   static const char *NPN_UserAgent(NPP instance) {
+    GGL_UNUSED(instance);
     CHECK_MAIN_THREAD();
     return kUserAgent;
   }
 
   static uint32 NPN_MemFlush(uint32 size) {
+    GGL_UNUSED(size);
     CHECK_MAIN_THREAD();
     return 0;
   }
@@ -591,6 +614,7 @@ class Plugin::Impl : public SmallObject<> {
   static void NPN_ReloadPlugins(NPBool reloadPages) {
     // We don't provide any plugin-in with the authority that can reload
     // all plug-ins in the plugins directory.
+    GGL_UNUSED(reloadPages);
     NOT_IMPLEMENTED();
   }
 
@@ -600,6 +624,7 @@ class Plugin::Impl : public SmallObject<> {
   }
 
   static jref NPN_GetJavaPeer(NPP instance) {
+    GGL_UNUSED(instance);
     NOT_IMPLEMENTED();
     return NULL;
   }
@@ -696,6 +721,8 @@ class Plugin::Impl : public SmallObject<> {
   }
 
   static void NPN_InvalidateRegion(NPP instance, NPRegion invalidRegion) {
+    GGL_UNUSED(instance);
+    GGL_UNUSED(invalidRegion);
     NOT_IMPLEMENTED();
   }
 
@@ -718,6 +745,7 @@ class Plugin::Impl : public SmallObject<> {
   static bool NPN_Invoke(NPP npp, NPObject *npobj, NPIdentifier method_name,
                          const NPVariant *args, uint32_t arg_count,
                          NPVariant *result) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->invoke &&
            npobj->_class->invoke(npobj, method_name, args, arg_count, result);
@@ -726,6 +754,7 @@ class Plugin::Impl : public SmallObject<> {
   static bool NPN_InvokeDefault(NPP npp, NPObject *npobj,
                                 const NPVariant *args, uint32_t arg_count,
                                 NPVariant *result) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->invokeDefault &&
            npobj->_class->invokeDefault(npobj, args, arg_count, result);
@@ -733,12 +762,17 @@ class Plugin::Impl : public SmallObject<> {
 
   static bool NPN_Evaluate(NPP npp, NPObject *npobj, NPString *script,
                            NPVariant *result) {
+    GGL_UNUSED(npp);
+    GGL_UNUSED(npobj);
+    GGL_UNUSED(script);
+    GGL_UNUSED(result);
     NOT_IMPLEMENTED();
     return false;
   }
 
   static bool NPN_GetProperty(NPP npp, NPObject *npobj,
                               NPIdentifier property_name, NPVariant *result) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->getProperty &&
            npobj->_class->getProperty(npobj, property_name, result);
@@ -747,6 +781,7 @@ class Plugin::Impl : public SmallObject<> {
   static bool NPN_SetProperty(NPP npp, NPObject *npobj,
                               NPIdentifier property_name,
                               const NPVariant *value) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->setProperty &&
            npobj->_class->setProperty(npobj, property_name, value);
@@ -754,6 +789,7 @@ class Plugin::Impl : public SmallObject<> {
 
   static bool NPN_RemoveProperty(NPP npp, NPObject *npobj,
                                  NPIdentifier property_name) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->removeProperty &&
            npobj->_class->removeProperty(npobj, property_name);
@@ -761,6 +797,7 @@ class Plugin::Impl : public SmallObject<> {
 
   static bool NPN_HasProperty(NPP npp, NPObject *npobj,
                               NPIdentifier property_name) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->hasProperty &&
            npobj->_class->hasProperty(npobj, property_name);
@@ -768,27 +805,34 @@ class Plugin::Impl : public SmallObject<> {
 
   static bool NPN_HasMethod(NPP npp, NPObject *npobj,
                             NPIdentifier method_name) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->hasMethod &&
            npobj->_class->hasMethod(npobj, method_name);
   }
 
   static void NPN_SetException(NPObject *npobj, const NPUTF8 *message) {
+    GGL_UNUSED(npobj);
+    GGL_UNUSED(message);
     NOT_IMPLEMENTED();
   }
 
   static bool NPN_PushPopupsEnabledState(NPP instance, NPBool enabled) {
+    GGL_UNUSED(instance);
+    GGL_UNUSED(enabled);
     NOT_IMPLEMENTED();
     return false;
   }
 
   static bool NPN_PopPopupsEnabledState(NPP instance) {
+    GGL_UNUSED(instance);
     NOT_IMPLEMENTED();
     return false;
   }
 
   static bool NPN_Enumerate(NPP npp, NPObject *npobj,
                             NPIdentifier **identifiers, uint32_t *count) {
+    GGL_UNUSED(npp);
     ENSURE_MAIN_THREAD(false);
     return npobj && npobj->_class && npobj->_class->enumerate &&
            npobj->_class->enumerate(npobj, identifiers, count);
@@ -799,10 +843,14 @@ class Plugin::Impl : public SmallObject<> {
     AsyncCall(void (*func)(void *), void *user_data)
         : func_(func), user_data_(user_data) { }
     virtual bool Call(MainLoopInterface *mainloop, int watch_id) {
+      GGL_UNUSED(mainloop);
+      GGL_UNUSED(watch_id);
       (*func_)(user_data_);
       return false;
     }
     virtual void OnRemove(MainLoopInterface *mainloop, int watch_id) {
+      GGL_UNUSED(mainloop);
+      GGL_UNUSED(watch_id);
       delete this;
     }
    private:
@@ -826,6 +874,7 @@ class Plugin::Impl : public SmallObject<> {
   // between calling this function and termination of the plugin instance.
   static void NPN_PluginThreadAsyncCall(NPP instance, void (*func)(void *),
                                         void *user_data) {
+    GGL_UNUSED(instance);
     if (GetGlobalMainLoop()->IsMainThread())
       DLOG("NPN_PluginThreadAsyncCall called from the non-main thread.");
     else
@@ -844,6 +893,7 @@ class Plugin::Impl : public SmallObject<> {
   // Only support window.top and window.location because the flash plugin
   // requires them.
   static bool BrowserWindowHasProperty(NPObject *npobj, NPIdentifier name) {
+    GGL_UNUSED(npobj);
     ENSURE_MAIN_THREAD(false);
     std::string name_str = GetIdentifierName(name);
     return name_str == "location" || name_str == "top";
@@ -866,6 +916,7 @@ class Plugin::Impl : public SmallObject<> {
   }
 
   static bool LocationHasMethod(NPObject *npobj, NPIdentifier name) {
+    GGL_UNUSED(npobj);
     ENSURE_MAIN_THREAD(false);
     return GetIdentifierName(name) == "toString";
   }
@@ -873,6 +924,8 @@ class Plugin::Impl : public SmallObject<> {
   static bool LocationInvoke(NPObject *npobj, NPIdentifier name,
                              const NPVariant *args, uint32_t argCount,
                              NPVariant *result) {
+    GGL_UNUSED(args);
+    GGL_UNUSED(argCount);
     ENSURE_MAIN_THREAD(false);
     if (GetIdentifierName(name) == "toString") {
       Impl *owner = reinterpret_cast<OwnedNPObject *>(npobj)->owner;
@@ -883,6 +936,7 @@ class Plugin::Impl : public SmallObject<> {
   }
 
   static bool LocationHasProperty(NPObject *npobj, NPIdentifier name) {
+    GGL_UNUSED(npobj);
     ENSURE_MAIN_THREAD(false);
     return GetIdentifierName(name) == "href";
   }
