@@ -51,6 +51,7 @@ static void MakeImageTransparent(QImage *img) {
 static void SetupPainter(QPainter *p) {
   p->setCompositionMode(QPainter::CompositionMode_SourceOver);
   p->setRenderHint(QPainter::SmoothPixmapTransform, true);
+  p->setRenderHint(QPainter::Antialiasing, false);
   p->setBackground(Qt::transparent);
 }
 
@@ -168,21 +169,29 @@ class QtCanvas::Impl {
                           const CanvasInterface *img,
                           double mx, double my,
                           const CanvasInterface *mask) {
+    GGL_UNUSED(mx);
+    GGL_UNUSED(my);
     DLOG("DrawCanvasWithMask: (%p, %p) on %p", img, mask, owner_);
     QPainter *p = painter_;
     const QtCanvas *s = reinterpret_cast<const QtCanvas*>(img);
     const QtCanvas *m = reinterpret_cast<const QtCanvas*>(mask);
-    QImage simg = *s->GetImage();
-    simg.setAlphaChannel(*m->GetImage());
-    Impl *impl = s->impl_;
+    QImage *simg = s->GetImage();
+    // FIXME: the content of img will be changed. but hopefully it's ok
+    // in current drawing model, where DrawCanvasWithMask() is only used in
+    // BasicElement's Draw() method and img is a temporary buffer.
+    QPainter *spainter = s->GetQPainter();
+    spainter->setCompositionMode(QPainter::CompositionMode_DestinationIn);
+    spainter->drawImage(0, 0, *m->GetImage());
+    spainter->setCompositionMode(QPainter::CompositionMode_SourceOver);
+    Impl *simpl = s->impl_;
     double sx, sy;
-    if (impl->GetScale(&sx, &sy) && image_) {
+    if (simpl->GetScale(&sx, &sy) && image_) {
       p->save();
       p->scale(sx, sy);
-      p->drawImage(QPointF(x / sx, y / sy), simg);
+      p->drawImage(QPointF(x / sx, y / sy), *simg);
       p->restore();
     } else {
-      p->drawImage(QPointF(x, y), simg);
+      p->drawImage(QPointF(x, y), *simg);
     }
     return true;
   }
@@ -297,6 +306,16 @@ class QtCanvas::Impl {
                            const CanvasInterface *texture,
                            Alignment align, VAlignment valign,
                            Trimming trimming, int text_flags) {
+    GGL_UNUSED(x);
+    GGL_UNUSED(y);
+    GGL_UNUSED(width);
+    GGL_UNUSED(height);
+    GGL_UNUSED(f);
+    GGL_UNUSED(texture);
+    GGL_UNUSED(align);
+    GGL_UNUSED(valign);
+    GGL_UNUSED(trimming);
+    GGL_UNUSED(text_flags);
     DLOG("DrawTextWithTexture: %s", text);
     ASSERT(0);
     return true;
@@ -305,9 +324,7 @@ class QtCanvas::Impl {
   bool IntersectRectClipRegion(double x, double y,
                                double w, double h) {
     if (w <= 0.0 || h <= 0.0) return false;
-    QPainter *p = painter_;
-    p->setClipping(true);
-    p->setClipRect(QRectF(x, y, w, h), Qt::IntersectClip);
+    painter_->setClipRect(QRectF(x, y, w, h), Qt::IntersectClip);
     return true;
   }
 
@@ -320,13 +337,9 @@ class QtCanvas::Impl {
   bool IntersectGeneralClipRegion(const ClipRegion &region) {
     QRegion qregion;
     region_ = &qregion;
-    QPainter *p = painter_;
 
     if (region.EnumerateRectangles(NewSlot(this, &Impl::IntersectRectangle))) {
-      p->setClipping(true);
-      p->setClipRegion(qregion);
-    } else {
-      p->setClipping(false);
+      painter_->setClipRegion(qregion, Qt::IntersectClip);
     }
     return true;
   }
@@ -483,6 +496,7 @@ bool QtCanvas::DrawCanvas(double x, double y, const CanvasInterface *img) {
 bool QtCanvas::DrawRawImage(double x, double y,
                             const char *data, RawImageFormat format,
                             int width, int height, int stride) {
+  GGL_UNUSED(stride);
   QImage::Format qt_format;
   if (format == RAWIMAGE_FORMAT_RGB24)
     qt_format = QImage::Format_RGB32;
@@ -564,7 +578,7 @@ QImage* QtCanvas::GetImage() const {
   return impl_->image_;
 }
 
-QPainter* QtCanvas::GetQPainter() {
+QPainter* QtCanvas::GetQPainter() const {
   return impl_->painter_;
 }
 
