@@ -159,7 +159,7 @@ TEST(UnicodeUtils, IsLegalString) {
   EXPECT_TRUE(IsLegalUTF16String(invalid_utf16_string, invalid_utf16_length));
 }
 
-TEST(UnicodeUtils, ConvertStringBuffer) {
+TEST(UnicodeUtils, ConvertString8To16Buffer) {
   UTF16Char buffer[200];
   std::string utf8(utf8_string);
   size_t output_length = 0;
@@ -195,6 +195,59 @@ TEST(UnicodeUtils, ConvertStringBuffer) {
               UTF16String(buffer, output_length));
 }
 
+TEST(UnicodeUtils, ConvertString16To8Buffer) {
+  char buffer[200];
+  std::string utf8(utf8_string);
+  size_t output_length = 0;
+  UTF16String utf16(utf16_string);
+
+  EXPECT_EQ(0U, ConvertStringUTF16ToUTF8Buffer(utf16, buffer, 0,
+                                               &output_length));
+  EXPECT_EQ(0U, output_length);
+
+  EXPECT_EQ(utf16.size(),
+            ConvertStringUTF16ToUTF8Buffer(utf16, buffer, utf8.size(),
+                                           &output_length));
+  EXPECT_EQ(utf8.size(), output_length);
+  EXPECT_EQ(utf8, std::string(buffer, output_length));
+
+  EXPECT_EQ(utf16.size(),
+            ConvertStringUTF16ToUTF8Buffer(utf16, buffer, 200, &output_length));
+  EXPECT_EQ(utf8.size(), output_length);
+  EXPECT_EQ(utf8, std::string(buffer, output_length));
+
+  EXPECT_EQ(utf16.size() - 2,
+            ConvertStringUTF16ToUTF8Buffer(utf16, buffer, utf8.size() - 1,
+                                           &output_length));
+  EXPECT_EQ(utf8.size() - 4, output_length);
+  EXPECT_EQ(utf8.substr(0, utf8.size() - 4),
+            std::string(buffer, output_length));
+
+  EXPECT_EQ(utf16.size() - 2,
+            ConvertStringUTF16ToUTF8Buffer(utf16, buffer, utf8.size() - 2,
+                                           &output_length));
+  EXPECT_EQ(utf8.size() - 4, output_length);
+  EXPECT_EQ(utf8.substr(0, utf8.size() - 4),
+            std::string(buffer, output_length));
+}
+
+TEST(UnicodeUtils, UTF16ToUTF8Converter) {
+  UTF16Char utf16[70];
+  char utf8[70];
+  for (int i = 0; i < 69; i++) {
+    utf16[i] = 'A';
+    utf8[i] = 'A';
+  }
+  utf16[69]  = 0;
+  utf8[69] = 0;
+
+  EXPECT_STREQ(utf8, UTF16ToUTF8Converter(utf16, 69).get());
+  for (int i = 69; i > 60; i--) {
+    EXPECT_EQ(std::string(utf8, i),
+              std::string(UTF16ToUTF8Converter(utf16, i).get()));
+  }
+}
+
 TEST(UnicodeUtils, DetectUTFEncoding) {
   std::string encoding("Garbage");
   EXPECT_FALSE(DetectUTFEncoding(std::string(""), &encoding));
