@@ -47,11 +47,22 @@
 #include "simple_gtk_host.h"
 #include "standalone_gtk_host.h"
 
+#ifndef GGL_GTK_HTML_SCRIPT_ENGINE
+#define GGL_GTK_HTML_SCRIPT_ENGINE "xulrunner"
+#endif
+
+#ifndef GGL_GTK_XML_HTTP_REQUEST
+#define GGL_GTK_XML_HTTP_REQUEST "curl"
+#endif
+
 using ggadget::Variant;
 
 static const char kOptionsName[] = "gtk-host-options";
 static const char kRunOnceSocketName[] = "ggl-host-socket";
 
+static const char kXHRExtensionSuffix[] = "-xml-http-request";
+
+// xml-http-request extension will be loaded separately.
 static const char *kGlobalExtensions[] = {
 // default framework must be loaded first, so that the default properties can
 // be overrode.
@@ -60,21 +71,33 @@ static const char *kGlobalExtensions[] = {
   "default-options",
   "dbus-script-class",
   "gtk-edit-element",
-  "gtkmoz-browser-element",
-  "gtk-flash-element",
   "gst-video-element",
   "gtk-system-framework",
   "gst-audio-framework",
 #ifdef GGL_HOST_LINUX
   "linux-system-framework",
 #endif
-  "smjs-script-runtime",
-  "curl-xml-http-request",
   "analytics-usage-collector",
   "google-gadget-manager",
   NULL
 };
 
+// extensions based on xulrunner.
+static const char *kXULRunnerExtensions[] = {
+  "smjs-script-runtime",
+  "gtkmoz-browser-element",
+  "gtk-flash-element",
+  NULL
+};
+
+// extensions based on webkit.
+static const char *kWebkitExtensions[] = {
+  "webkit-script-runtime",
+  "gtkwebkit-browser-element",
+  "html-flash-element",
+  NULL
+};
+
 static const char kHelpString[] =
   "Google Gadgets for Linux " GGL_VERSION
   " (Gadget API version " GGL_API_VERSION ")\n"
@@ -87,6 +110,9 @@ static const char kHelpString[] =
   "      1 - Draw bounding boxes around container elements.\n"
   "      2 - Draw bounding boxes around all elements.\n"
   "      4 - Draw bounding boxes around clip region.\n"
+  "  -pp, --private-profile\n"
+  "      Uses a private profile to start " GGL_APP_NAME ", so that\n"
+  "      multiple application instances can be started at the same time.\n"
 #endif
   "  -b, --border\n"
   "      Draw window border for Main View.\n"
@@ -116,6 +142,13 @@ static const char kHelpString[] =
   "      Disable the usage collector\n"
   "  -gp, --grant-permissions\n"
   "      Grant all permissions required by gadgets silently.\n"
+  "  -hs, --html-script-engine\n"
+  "      Specify html/script engine, default: " GGL_GTK_HTML_SCRIPT_ENGINE ".\n"
+  "      Available engines: xulrunner, webkit.\n"
+  "  -xhr, --xml-http-request\n"
+  "      Specify xml-http-request extension to load, default: "
+         GGL_GTK_XML_HTTP_REQUEST ".\n"
+  "      Available extensions: curl, soup.\n"
   "  -h, --help\n"
   "      Print this message and exit.\n"
   "\n"
@@ -126,6 +159,7 @@ static const char kHelpString[] =
 
 enum ArgumentID {
   ARG_DEBUG = 1,
+  ARG_PRIVATE_PROFILE,
   ARG_BORDER,
   ARG_NO_TRANSPARENT,
   ARG_NO_DECORATOR,
@@ -138,12 +172,15 @@ enum ArgumentID {
   ARG_DEBUG_CONSOLE,
   ARG_NO_COLLECTOR,
   ARG_GRANT_PERMISSIONS,
+  ARG_HTML_SCRIPT_ENGINE,
+  ARG_XML_HTTP_REQUEST,
   ARG_HELP
 };
 
 static const ggadget::HostArgumentInfo kArgumentsInfo[] = {
 #ifdef _DEBUG
   { ARG_DEBUG,             Variant::TYPE_INT64, "-d",  "--debug" },
+  { ARG_PRIVATE_PROFILE,   Variant::TYPE_BOOL,  "-pp", "--private-profile" },
 #endif
   { ARG_BORDER,            Variant::TYPE_BOOL,  "-b",  "--border" },
   { ARG_NO_TRANSPARENT,    Variant::TYPE_BOOL,  "-nt", "--no-transparent" },
@@ -157,6 +194,8 @@ static const ggadget::HostArgumentInfo kArgumentsInfo[] = {
   { ARG_DEBUG_CONSOLE,     Variant::TYPE_INT64, "-dc", "--debug-console" },
   { ARG_NO_COLLECTOR,      Variant::TYPE_BOOL,  "-nc", "--no-collector" },
   { ARG_GRANT_PERMISSIONS, Variant::TYPE_BOOL,  "-gp", "--grant-permissions" },
+  { ARG_HTML_SCRIPT_ENGINE,Variant::TYPE_STRING,"-hs", "--html-script-engine" },
+  { ARG_XML_HTTP_REQUEST,  Variant::TYPE_STRING,"-xhr","--xml-http-request" },
   { ARG_HELP,              Variant::TYPE_BOOL,  "-h",  "--help" },
   { -1,                    Variant::TYPE_VOID, NULL, NULL } // End of list
 };
@@ -180,7 +219,9 @@ struct Arguments {
 #endif
       debug_console(ggadget::Gadget::DEBUG_CONSOLE_DISABLED),
       no_collector(false),
-      grant_permissions(false) {
+      grant_permissions(false),
+      html_script_engine(GGL_GTK_HTML_SCRIPT_ENGINE),
+      xml_http_request(GGL_GTK_XML_HTTP_REQUEST) {
   }
 
   int debug_mode;
@@ -196,6 +237,8 @@ struct Arguments {
   ggadget::Gadget::DebugConsoleConfig debug_console;
   bool no_collector;
   bool grant_permissions;
+  std::string html_script_engine;
+  std::string xml_http_request;
 };
 
 static ggadget::HostArgumentParser g_argument_parser(kArgumentsInfo);
@@ -242,6 +285,12 @@ static void ExtractArgumentsValue() {
     g_arguments.no_collector = ggadget::VariantValue<bool>()(arg_value);
   if (g_argument_parser.GetArgumentValue(ARG_GRANT_PERMISSIONS, &arg_value))
     g_arguments.grant_permissions = ggadget::VariantValue<bool>()(arg_value);
+  if (g_argument_parser.GetArgumentValue(ARG_HTML_SCRIPT_ENGINE, &arg_value))
+    g_arguments.html_script_engine =
+        ggadget::VariantValue<std::string>()(arg_value);
+  if (g_argument_parser.GetArgumentValue(ARG_XML_HTTP_REQUEST, &arg_value))
+    g_arguments.xml_http_request =
+        ggadget::VariantValue<std::string>()(arg_value);
 }
 
 static void OnHostExit(ggadget::HostInterface *host) {
@@ -398,17 +447,6 @@ int main(int argc, char* argv[]) {
   // object lives longer than any other objects, including the static objects.
   ggadget::SetGlobalMainLoop(new ggadget::gtk::MainLoop());
 
-  std::string profile_dir =
-      ggadget::BuildFilePath(ggadget::GetHomeDirectory().c_str(),
-                             ggadget::kDefaultProfileDirectory, NULL);
-  ggadget::EnsureDirectories(profile_dir.c_str());
-
-  ggadget::RunOnce run_once(
-      ggadget::BuildFilePath(profile_dir.c_str(),
-                             kRunOnceSocketName,
-                             NULL).c_str());
-  run_once.ConnectOnMessage(ggadget::NewSlot(OnClientMessage));
-
   // Parse command line.
   if (argc > 1) {
     g_argument_parser.Start();
@@ -425,6 +463,24 @@ int main(int argc, char* argv[]) {
     return 0;
   }
 
+  std::string profile_dir =
+      ggadget::BuildFilePath(ggadget::GetHomeDirectory().c_str(),
+                             ggadget::kDefaultProfileDirectory, NULL);
+#ifdef _DEBUG
+  // Check --private-profile argument.
+  if (g_argument_parser.GetArgumentValue(ARG_PRIVATE_PROFILE, NULL)) {
+    profile_dir = ggadget::StringPrintf("%s-%u", profile_dir.c_str(), getpid());
+  }
+#endif
+
+  ggadget::EnsureDirectories(profile_dir.c_str());
+
+  ggadget::RunOnce run_once(
+      ggadget::BuildFilePath(profile_dir.c_str(),
+                             kRunOnceSocketName,
+                             NULL).c_str());
+  run_once.ConnectOnMessage(ggadget::NewSlot(OnClientMessage));
+
   // If another instance is already running, then send all arguments to it.
   if (run_once.IsRunning()) {
     gdk_notify_startup_complete();
@@ -455,10 +511,27 @@ int main(int argc, char* argv[]) {
       ggadget::ExtensionManager::CreateExtensionManager();
   ggadget::ExtensionManager::SetGlobalExtensionManager(ext_manager);
 
+  // Load xml-http-request extension first.
+  std::string xhr_extension;
+  if (g_arguments.xml_http_request.length()) {
+    xhr_extension = g_arguments.xml_http_request + kXHRExtensionSuffix;
+  } else {
+    xhr_extension = std::string(GGL_GTK_XML_HTTP_REQUEST) + kXHRExtensionSuffix;
+  }
+  ext_manager->LoadExtension(xhr_extension.c_str(), false);
+
   // Ignore errors when loading extensions.
   for (size_t i = 0; kGlobalExtensions[i]; ++i)
     ext_manager->LoadExtension(kGlobalExtensions[i], false);
 
+  if (g_arguments.html_script_engine == "xulrunner") {
+    for (size_t i = 0; kXULRunnerExtensions[i]; ++i)
+      ext_manager->LoadExtension(kXULRunnerExtensions[i], false);
+  } else if (g_arguments.html_script_engine == "webkit") {
+    for (size_t i = 0; kWebkitExtensions[i]; ++i)
+      ext_manager->LoadExtension(kWebkitExtensions[i], false);
+  }
+
   // Register JavaScript runtime.
   ggadget::ScriptRuntimeManager *script_runtime_manager =
       ggadget::ScriptRuntimeManager::get();
